// Client side test program for accept failure
// This program forks itself to do the actual protocol exchanges, which
// tickles an MPICH bug
// exercise the client-server protocol with these commands:
// do nothing with a "NOOP" message
// report process name when receive a "NAME" message
// end this client with an "END" message
// stop the server with a "STOP" message

// master process simply forks and wait with appropriate routines to do a few
// connections.

#include <mpi.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>

#include "commands.h"

MPI::Intracomm doConnect();
void doJob();
void doStopServer();

char pname[MPI_MAX_PROCESSOR_NAME];
int lpname;

int main(int argc, char **argv)
{

    pid_t pid;

    pid = fork();
    if(pid == 0) {                      // child
        doJob();
        exit(0);
    } else {
        int status;
        waitpid(pid, &status, 0);
    }

    pid = fork();
    if(pid == 0) {                      // child
        doJob();
        exit(0);
    } else {
        int status;
        waitpid(pid, &status, 0);
    }

    pid = fork();
    if(pid == 0) {                      // child
        doStopServer();                 // last job: stop the server
        exit(0);
    } else {
        int status;
        waitpid(pid, &status, 0);
    }

    return(0);
}

MPI::Intracomm doConnect()
{
    char serverPort[ MPI_MAX_PORT_NAME ];

    MPI::Init_thread( 3 );

    printf("client lookup\n");
    MPI::Lookup_name( serviceName, MPI_INFO_NULL, serverPort );

    printf("client connect %s\n", serverPort);
    MPI::Intercomm clientComm;		// result of Connect() call
    clientComm = MPI::COMM_SELF.Connect( serverPort, MPI_INFO_NULL, 0);

    MPI::Intracomm userComm;        // result of Merge() call
    userComm = clientComm.Merge( 0 ); // server [our end] is high
    clientComm.Disconnect();

    return userComm;
}

void doJob( )
{
    MPI::Intracomm comm = doConnect();
    printf("client request END\n");
    int request = END;
    comm.Bcast( &request, 1, MPI::INT, 0); // client command
    printf("client disconnect\n");
    comm.Disconnect();
    printf("client finalize\n");
    MPI::Finalize();
}

void doStopServer( )
{
    MPI::Intracomm comm = doConnect();
    printf("client request STOP\n");
    int request = STOP;
    comm.Bcast( &request, 1, MPI::INT, 0); // client command
    printf("client disconnect\n");
    comm.Disconnect();
    printf("client finalize\n");
    MPI::Finalize();
}

