// Server side test program for accept failure
// publush a name, accept connections/close connections in a loop
// report process name when receive a "NAME" message
// go back to accept when receive an "END" message
// exit when the loop receives a "STOP" message.

#include <mpi.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <stdio.h>

#include "commands.h"

bool processCommands( MPI::Intracomm );

char pname[ MPI_MAX_PROCESSOR_NAME ];
int lpname;

int main(int argc, char **argv)
{
    char serverPort[ MPI_MAX_PORT_NAME ];

    MPI::Init_thread( 3 );
    MPI::Get_processor_name( pname, lpname );

    if( MPI::COMM_WORLD.Get_rank() == 0 ) {
        MPI::Open_port(MPI_INFO_NULL, serverPort);
        MPI::Publish_name( serviceName,  MPI_INFO_NULL, serverPort );
    } // lead server
    printf("server publish\n");
    MPI::COMM_WORLD.Bcast( serverPort, MPI_MAX_PORT_NAME, MPI::BYTE, 0);

    bool done = false;

    while(!done) {
        MPI::Intracomm acceptComm;      // MPI Group for new server connects
        acceptComm = MPI::COMM_WORLD.Dup( ); // Intracomm for new connections

        printf("server accept on %s\n", serverPort);
        MPI::Intercomm clientComm;		// result of Accept() call
        clientComm = acceptComm.Accept(serverPort, MPI_INFO_NULL, 0);

        acceptComm.Disconnect( );
        acceptComm = MPI_COMM_NULL;

        MPI::Intracomm userComm;        // result of Merge() call
        userComm = clientComm.Merge( 1 ); // server [our end] is high

        clientComm.Disconnect();
        clientComm = MPI_COMM_NULL;

        done = processCommands( userComm );
        userComm.Disconnect();
    }
    printf("server unpublish\n");
    MPI::Unpublish_name( serviceName,  MPI_INFO_NULL, serverPort );
    printf("server finalize\n");
    MPI::Finalize();
    return(0);
}

// return true when it's time to shut down the server

bool processCommands( MPI::Intracomm comm )
{
    bool result = false;
    bool done = false;
    while(!done) {
        int request;
        printf("server get request\n");
        comm.Bcast( &request, 1, MPI::INT, 0); // client command

        switch(request) {
        case NOOP:
            printf("server got NOOP\n");
            break;
        case NAME:
            printf("server got NAME\n");
            comm.Send(pname, MPI_MAX_PROCESSOR_NAME, MPI::BYTE, 0, 99);
            break;
        case END:
            printf("server got END\n");
            done = true;
            break;
        case STOP:
            printf("server got STOP\n");
            result = true;
            done = true;
            break;
        default:
            break;
        }
    }
    return result;
}

